#include "abuleducamerawidgetv1.h"

AbulEduCameraWidgetV1::AbulEduCameraWidgetV1(QWidget *parent) :
    QWidget(parent),
    m_image(0),
    m_scribbling(false),
    m_selectingAllowed(false),
    m_drawRect(false)
{
    m_selectedRectColor = QColor(Qt::blue);
    m_selectedRectPenWidth = 3;
    m_selectedRectPen = QPen(m_selectedRectColor, m_selectedRectPenWidth,
                             Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    m_startPoint = QPoint(0,0);
    m_endPoint   = QPoint(0,0);

}

void AbulEduCameraWidgetV1::abeCameraWidgetSetImage(const QImage &image)
{
    m_image = image;
    m_scaledImage = m_image.scaled(size(), Qt::KeepAspectRatio);
    m_ratio = (qreal)m_scaledImage.width() / (qreal)m_image.width();
    m_targetOrigin = getTargetOrigin();
    update();
}

void AbulEduCameraWidgetV1::abeCameraWidgetSetSelectingAllowed(bool selecting)
{
    m_selectingAllowed = selecting;
}

void AbulEduCameraWidgetV1::abeCameraWidgetSetSelectedRectColor(QColor color)
{
        m_selectedRectColor = color;
        m_selectedRectPen = QPen(m_selectedRectColor, m_selectedRectPenWidth,
                                 Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
}

void AbulEduCameraWidgetV1::abeCameraWidgetSetSelectedPenWidth(int penWidth)
{
    m_selectedRectPenWidth = penWidth;
    m_selectedRectPen = QPen(m_selectedRectColor, m_selectedRectPenWidth,
                             Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
}

void AbulEduCameraWidgetV1::abeCameraWidgetSetRectColor(QColor color)
{
    m_rectColor = color;
    m_rectPen = QPen(m_rectColor, m_rectPenWidth,
                     Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
}

void AbulEduCameraWidgetV1::abeCameraWidgetSetRectPenWidth(int penWidth)
{
    m_rectPenWidth = penWidth;
    m_rectPen = QPen(m_rectColor, m_rectPenWidth,
                             Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
}

void AbulEduCameraWidgetV1::abeCameraWidgetDrawRect(QRect rect)
{
    m_rect = rect;
    /* On calcule les coordonnées du rectangle en fonction du coef de réduction de l'image */
    m_rect.setX((qreal)rect.x()*m_ratio+getTargetOrigin().x());
    m_rect.setY((qreal)rect.y()*m_ratio+getTargetOrigin().y());
    m_rect.setWidth((qreal)rect.width()*m_ratio);
    m_rect.setHeight((qreal)rect.height()*m_ratio);

    /* Si la largeur ou la hauteur sont nulles, on ne dessine pas le rectangle */
    m_drawRect = rect.width()*rect.height();
}


void AbulEduCameraWidgetV1::mousePressEvent(QMouseEvent *event)
{
    m_scribbling = false;
    if (event->button() == Qt::LeftButton && getTargetRect().contains(event->pos()))
    {
        m_startPoint = event->pos();
        m_endPoint = m_startPoint;        
        m_selectedRect = QRect(m_startPoint, m_endPoint);
        if(m_selectingAllowed)
        {
            m_scribbling = true;
        }
    }
}

void AbulEduCameraWidgetV1::mouseMoveEvent(QMouseEvent *event)
{
    if ((event->buttons() & Qt::LeftButton)
            && m_scribbling
            && getTargetRect().contains(event->pos()))
    {
        m_endPoint = event->pos();
        /* POur ne pas sortir de l'image */
        if(m_endPoint.x() < 0)
        {
            m_endPoint.setX(0);
        }
        else if(m_endPoint.x() > m_scaledImage.width() + getTargetOrigin().x()-4 )
        {
            m_endPoint.setX(m_scaledImage.width() + getTargetOrigin().x()-4);
        }

        if(m_endPoint.y() < 0)
        {
            m_endPoint.setY(0);
        }
        else if(m_endPoint.y() > m_scaledImage.height() + getTargetOrigin().y()-4 )
        {
            m_endPoint.setY(m_scaledImage.height() + getTargetOrigin().y()-4);
        }

        m_selectedRect = QRect(m_startPoint, m_endPoint).normalized();
        /* On redessine uniquement la zone sélectionnée */
        int rad = (m_selectedRectPenWidth / 2) + 2;
        update(QRect(m_selectedRect)
               .adjusted(-rad, -rad, +rad, +rad));
        /* On informe du nouveau ROI en train d'etre dessiné */
        emit abeSignalCameraWidgetSelectingROI(getNewROI());
    }
}

void AbulEduCameraWidgetV1::mouseReleaseEvent(QMouseEvent *event)
{
    Q_UNUSED(event)
    QRect rect = getNewROI();
    /* Si le rectangle est trop petit, on le remet à zéro */
    if(rect.width() > 100 || rect.height() > 100)
    {
        emit abeSignalCameraWidgetNewROI(rect);
    }
    else
    {
        emit abeSignalCameraWidgetNewROI(QRect(0,0,0,0));
    }
    m_scribbling = false;
}

void AbulEduCameraWidgetV1::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)
    QPainter painter(this);
    painter.drawImage(m_targetOrigin, m_scaledImage);

    if(m_scribbling)
    {
        painter.setPen(m_selectedRectPen);
        painter.drawRect(m_selectedRect);
    }
    if(m_drawRect)
    {
        painter.setPen(m_rectPen);
        painter.drawRect(m_rect);
    }
}

void AbulEduCameraWidgetV1::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event)
    m_scaledImage = m_image.scaled(size(), Qt::KeepAspectRatio);
    m_ratio = (qreal)m_scaledImage.width() / (qreal)m_image.width();
    emit abeSignalCameraWidgetRatioChanged(m_ratio);

    m_targetOrigin = getTargetOrigin();
    m_startPoint = m_endPoint;
    m_scribbling = false;
}

QPoint AbulEduCameraWidgetV1::getTargetOrigin()
{
    int x = (width()  - m_scaledImage.width())/2;
    int y = (height() - m_scaledImage.height())/2;
    return QPoint(x, y);
}

QRect AbulEduCameraWidgetV1::getTargetRect()
{
    return QRect(getTargetOrigin(), m_scaledImage.size());
}

QRect AbulEduCameraWidgetV1::getNewROI()
{
    /* Calcule les coordonnées du ROI par rapport à l'image originale */
    int X1 = (qreal)(m_startPoint.x() - getTargetOrigin().x()) / m_ratio;
    int Y1 = (qreal)(m_startPoint.y() - getTargetOrigin().y()) / m_ratio;
    int X2 = (qreal)(m_endPoint.x()   - getTargetOrigin().x()) / m_ratio;
    int Y2 = (qreal)(m_endPoint.y()   - getTargetOrigin().y()) / m_ratio;
    return QRect(QPoint(X1, Y1), QPoint(X2, Y2)).normalized();
}
