#ifndef ABULEDUCAMERAWIDGETV1_H
#define ABULEDUCAMERAWIDGETV1_H

#include <QWidget>
#include <QImage>
#include <QPainter>
#include <QPaintEvent>
#include <QDebug>
#include <QColor>

class AbulEduCameraWidgetV1 : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(bool selectingAllowed READ abeCameraWidgetIsSelectingAllowed WRITE abeCameraWidgetSetSelectingAllowed MEMBER m_selectingAllowed)

    Q_PROPERTY(QColor selectedRectColor READ abeCameraWidgetGetSelectedRectColor WRITE abeCameraWidgetSetSelectedRectColor MEMBER m_selectedRectColor)
    Q_PROPERTY(int selectedPenWidth READ abeCameraWidgetGetSelectedPenWidth WRITE abeCameraWidgetSetSelectedPenWidth MEMBER m_selectedRectPenWidth)

    Q_PROPERTY(QColor rectColor READ abeCameraWidgetGetRectColor WRITE abeCameraWidgetSetRectColor MEMBER m_rectColor)
    Q_PROPERTY(int rectPenWidth READ abeCameraWidgetGetRectPenWidth WRITE abeCameraWidgetSetRectPenWidth MEMBER m_rectPenWidth)
    public:
    explicit AbulEduCameraWidgetV1(QWidget *parent = 0);
    ///
    /// \brief abeCameraWidgetSetImage affiche la QImage passée en paramètre
    /// \param image QImage a afficher
    ///
    void abeCameraWidgetSetImage(const QImage &image);

    ///
    /// \brief abeCameraWidgetSetSelectingAllowed On peut faire une sélection à la souris
    /// \param selecting
    ///
    void abeCameraWidgetSetSelectingAllowed(bool selecting = true);

    ///
    /// \brief abeCameraWidgetIsSelectingAllowed
    /// \return La sélection est autorisée (true) ou non (false)
    ///
    bool abeCameraWidgetIsSelectingAllowed(){ return m_selectingAllowed;}

    ///
    /// \brief abeCameraWidgetGetSelectedRectColor retourne la couleur du rectangle de sélection
    /// \return La couleur (QColor)
    ///
    QColor abeCameraWidgetGetSelectedRectColor(){return m_selectedRectColor;}

    ///
    /// \brief abeCameraWidgetSetSelectedRectColor détermine la couleur du rectangle sélectionné
    /// \param color La couleur du tracé QColor (bleue par défaut)
    ///
    void abeCameraWidgetSetSelectedRectColor(QColor color = Qt::blue);

    ///
    /// \brief abeCameraWidgetGetSelectedPenWidth l'épaisseur du contour du rectangle sélectionné
    /// \return l'épaisseur du trait de contour en pixels
    ///
    int abeCameraWidgetGetSelectedPenWidth(){return m_selectedRectPenWidth;}

    ///
    /// \brief abeCameraWidgetSetSelectedPenWidth détermine l'épaisseur du contour du rectangle sélectionné
    /// \param penWidth L'épaisseur du trait en pixels (2 par défaut)
    ///
    void abeCameraWidgetSetSelectedPenWidth(int penWidth = 2);

    ///
    /// \brief abeCameraWidgetDrawRect dessine un rectangle sur l'image
    ///
    /// \note Ce n'est pas le rectangle de sélection, mais un rectangle que l'on ajoute par programmation
    /// \param rect Le QRect à dessiner
    /// \note Les coordonnées du rectangle sont à donner par rapport à l'image d'origine.
    ///
    void abeCameraWidgetDrawRect(QRect rect);

    ///
    /// \brief abeCameraWidgetGetRectColor retourne la couleur du rectangle à dessiner
    /// \return La couleur (QColor)
    ///
    QColor abeCameraWidgetGetRectColor(){return m_rectColor;}

    ///
    /// \brief abeCameraWidgetSetRectColor détermine la couleur du rectangle à dessiner
    /// \param color La couleur du tracé QColor (bleue par défaut)
    ///
    void abeCameraWidgetSetRectColor(QColor color = Qt::blue);

    ///
    /// \brief abeCameraWidgetGetRectPenWidth l'épaisseur du contour du rectangle à dessiner
    /// \return l'épaisseur du trait de contour en pixels
    ///
    int abeCameraWidgetGetRectPenWidth(){return m_rectPenWidth;}

    ///
    /// \brief abeCameraWidgetSetRectPenWidth détermine l'épaisseur du contour du rectangle à dessiner
    /// \param penWidth L'épaisseur du trait en pixels (2 par défaut)
    ///
    void abeCameraWidgetSetRectPenWidth(int penWidth = 2);


private:
    // Le point du widget à partir duquel on dessine l'image (pour la centrer)
    QPoint getTargetOrigin();
    // Le rect de l'image dans le widget
    QRect  getTargetRect();
    // Le rect de la zone sélectionnée dans les coordonnées de l'image originale
    QRect  getNewROI();

    QImage m_image;
    QImage m_scaledImage;
    qreal  m_ratio;
    QPoint m_targetOrigin;
    QRect  m_targetRect;
    QPoint m_startPoint;
    QPoint m_endPoint;
    bool   m_scribbling;
    bool   m_selectingAllowed;
    QColor m_selectedRectColor;
    int    m_selectedRectPenWidth;
    QPen   m_selectedRectPen;
    QRect  m_selectedRect;
    QRect  m_rect;
    QColor m_rectColor;
    int    m_rectPenWidth;
    QPen   m_rectPen;
    bool   m_drawRect;
signals:
    ///
    /// \brief abeSignalCameraWidgetNewROI retourne la zone sélectionnée dans l'image
    ///
    /// Le signal est émis sur le mouseRelease
    /// \param rect les valeurs du rect sont données par rapport à l'image originale.
    ///
    void abeSignalCameraWidgetNewROI(QRect rect);

    ///
    /// \brief abeSignalCameraWidgetSelectingROI retourne la zone en cours de sélestion dans l'image
    ///
    /// Le signal est émis sur le mouseMove
    /// \param rect les valeurs du rect sont données par rapport à l'image originale.
    ///
    void abeSignalCameraWidgetSelectingROI(QRect rect);

    ///
    /// \brief abeSignalCameraWidgetRatioChanged informe du redimensionnement de l'image
    /// \param ratio Le nouveau rapport d'agrandissement
    ///
    void abeSignalCameraWidgetRatioChanged(qreal ratio);

public slots:

protected slots:
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void paintEvent(QPaintEvent *event);
    void resizeEvent(QResizeEvent *event);

};

#endif // ABULEDUCAMERAWIDGETV1_H
