/** Classe de base pour les mainwindows des logiciels AbulEdu (Abuleducamera)
  *
  * @see https://redmine.ryxeo.com/projects/abuledu-abuleducamera
  * @author 2014 Jean-Louis Frucot<frucot.jeanlouis@free.fr>
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/UtilitaireMainWindow
  * @author 2012 Philippe <philippe.cadaugade@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "utilitairemainwindow.h"
#include "ui_utilitairemainwindow.h"

UtilitaireMainWindow::UtilitaireMainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::UtilitaireMainWindow),
    m_camera(0),
    m_cameraWorker(0),
    m_camThread(0)
{
    setAttribute(Qt::WA_DeleteOnClose);

    qRegisterMetaType< AbulEduCameraWorker::ImageTypes >("ImagesTypes");// Permet de passer des ImagesTypes dans les signaux
    /* Supprime la bordure */
//    setWindowFlags(Qt::CustomizeWindowHint);

    ui->setupUi(this);
    /* Initialisation des variables */
    m_localDebug = 0;

    m_abuledufile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);

    m_previousWidget = ui->pagePrincipale;

    /* On passe un abuledufile a la boite de gestion des fichiers */
    ui->widgetAbulEduBoxFileManager->abeSetFile(m_abuledufile);
    connect(ui->widgetAbulEduBoxFileManager, SIGNAL(signalAbeFileSelected(QSharedPointer<AbulEduFileV1>)), this, SLOT(slotOuvrirFichier(QSharedPointer<AbulEduFileV1>)), Qt::UniqueConnection);
    connect(ui->widgetAbulEduBoxFileManager, SIGNAL(signalAbeFileCloseOrHide()), this, SLOT(slotAfficheWidgetPrincipal()), Qt::UniqueConnection);
    connect(ui->widgetAbulEduBoxFileManager, SIGNAL(signalAbeFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation,QString,bool)), this, SLOT(slotFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation,QString,bool)), Qt::UniqueConnection);

    /* Toute l'infra de gestion réseau */
    m_nam = abeApp->getAbeNetworkAccessManager();
    connect(m_nam,SIGNAL(ssoAuthStatus(int)),this,SLOT(slotSSOStatusChanged(int)),Qt::UniqueConnection);

    /* La boite à propos, même en version V1, n'est pas une lib externe, je ne l'inclus donc pas pour l'instant */

    /* On centre la fenetre sur l'ecran de l'utilisateur si elle n'est pas fullscreen */
    if(windowState() != Qt::WindowMaximized && windowState() != Qt::WindowFullScreen) {
        QDesktopWidget *widget = QApplication::desktop();
        int desktop_width = widget->width();
        int desktop_height = widget->height();
        this->move((desktop_width-this->width())/2, (desktop_height-this->height())/2);
    }
    setTitle(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus());

    ///////////////////////////////////////////////////////////////////////////////////////
    ///                         My project
    ///////////////////////////////////////////////////////////////////////////////////////

}

UtilitaireMainWindow::~UtilitaireMainWindow()
{
    delete ui;
    m_abuledufile->deleteLater();
    if(m_cameraWorker)
    {
        m_cameraWorker->deleteLater();
    }
    if(m_camThread)
    {
        m_camThread->quit();
        m_camThread->wait();
    }
}

///////////////////////////////////////////////////////////////////////////////////////
///                         My project
///////////////////////////////////////////////////////////////////////////////////////


void UtilitaireMainWindow::createCameraController()
{
    m_camera = new AbulEduCameraControllerV1(this);
    /* Permet de recevoir l'image de la zone sélectionnée */
    connect(m_camera, SIGNAL(signalCroppedImageReady(QImage)), this, SLOT(slotCroppedQImageFromCamReceived(QImage)));
    /* Permet de recevoir l'image entière */
    connect(m_camera, SIGNAL(signalFullImageReady(QImage)),    this, SLOT(slotFullQImageFromCamReceived(QImage)));
    /* Récupère les codes d'erreur s'il y a lieu (0 -> tout va bien) */
    connect(m_camera, SIGNAL(signalAbeCameraError(int)), this, SLOT(slotCameraErrorHandler(int)));
    /* Récupère la taille des images capturées */
    connect(m_camera, SIGNAL(signalAbeCameraImageSize(int,int)), this, SLOT(slotGetImageSize(int,int)));
    /* Récupère le nombre d'images par seconde capturées */
    connect(m_camera, SIGNAL(signalCameraFPSAverage(int)), this, SLOT(slotCameraFPSHandler(int)));
    /* Passe directement les dimensions de la zone sélectionnée à la classe AbulEduCameraControllerV1 */
    connect(ui->wgCameraView, SIGNAL(abeSignalCameraWidgetNewROI(QRect)), m_camera, SIGNAL(signalSetROI(QRect)));
    /* Récupère les dimensions de la zone sélectionnée */
    connect(ui->wgCameraView, SIGNAL(abeSignalCameraWidgetNewROI(QRect)),       this, SLOT(drawRect(QRect)));
    /* Récupère les dimensions de la zone en cours de sélection */
    connect(ui->wgCameraView, SIGNAL(abeSignalCameraWidgetSelectingROI(QRect)), this, SLOT(drawRect(QRect)));

    /* Passe au AbulEduCameraControllerV1 les types d'images voulus */
    m_camera->abeCameraSetImagesTypes(AbulEduCameraControllerV1::FullQImage |
                                      AbulEduCameraControllerV1::CroppedImage);
}





///////////////////////////////////////////////////////////////////////////////////////
///                         Gestion de l'interface (juste pour l'exemple)
///////////////////////////////////////////////////////////////////////////////////////
void UtilitaireMainWindow::on_sbDeviceNumber_valueChanged(int deviceNumber)
{
    /* On change de caméra */
    if(!m_camera)
    {
        createCameraController();
    }
    else
    {
        m_camera->abeCameraStopCapture();
    }
    m_camera->abeCameraInitCamera(deviceNumber);
}

void UtilitaireMainWindow::on_cbResolution_currentIndexChanged(int index)
{
    /* On change la résolution de la caméra. */
    if(!m_camera)
    {
        createCameraController();
        m_camera->abeCameraInitCamera(ui->sbDeviceNumber->value());
    }
    else
    {
        m_camera->abeCameraStopCapture();
    }
    switch(index)
    {
    case 0:
        m_camera->abeCameraSetFrameResolution(640, 400);
        break;
    case 1:
        m_camera->abeCameraSetFrameResolution(1024, 768);
        break;
    case 2:
        m_camera->abeCameraSetFrameResolution(1280, 720);
        break;
    case 3:
        m_camera->abeCameraSetFrameResolution(1920, 1080);
        break;
    case 4:
        m_camera->abeCameraSetFrameResolution(9999, 9999);
        break;
    }
}

void UtilitaireMainWindow::on_hsFPS_valueChanged(int value)
{
    /* On change la vitesse de capture */
    if(!m_camera)
    {
        createCameraController();
        m_camera->abeCameraInitCamera(ui->sbDeviceNumber->value());
    }
    else
    {
        m_camera->abeCameraStopCapture();
    }
    ui->label->setText(QString::number(int(value))+" FPS");
    m_camera->abeCameraSetFrameRefreshInterval(int(1000/value));
}

void UtilitaireMainWindow::slotCroppedQImageFromCamReceived(QImage image)
{
    /* On affiche l'image reçue sur le widget central */
    ui->wgCameraView->abeCameraWidgetSetImage(image);
    /* On informe le AbulEduCameraControllerV1 qu'on est prêt à recevoir une autre image */
    m_camera->slotCameraControllerReadyToGetNewImage(true);
}

void UtilitaireMainWindow::slotFullQImageFromCamReceived(QImage image)
{
    /* Affiche l'image complète dans le petit widget en bas à droite */
    ui->camFullView->abeCameraWidgetSetImage(image);

}

void UtilitaireMainWindow::slotGetImageSize(int width, int height)
{
    ui->lbSize->setText(QString::number(width)+"x"+QString::number(height)+" px");
}

void UtilitaireMainWindow::slotCameraErrorHandler(int errorNumber)
{
    switch (errorNumber)
    {
    case 0:
//        ui->lbCameraView->setText(trUtf8("Pas d'erreur' : La caméra est connectée. \n Cliquez sur Démarrer !"));
        break;
    case 1:
//        ui->lbCameraView->setText(trUtf8("Erreur 1 : La caméra n'a pu être connectée"));
        break;
    default:
        break;
    }
}

void UtilitaireMainWindow::slotCameraFPSHandler(int fpsAverage)
{
    ui->lbFPS->setText(QString::number(fpsAverage)+" fps");
}

void UtilitaireMainWindow::on_btnCapture_clicked()
{
    if(!m_camera)
    {
        createCameraController();
        m_camera->abeCameraInitCamera(ui->sbDeviceNumber->value());
    }
    else
    {
        m_camera->abeCameraStopCapture();
    }
    m_camera->abeCameraSetFrameRefreshInterval(int(1000/ui->hsFPS->value()));
    m_camera->abeCameraStartCapture();
}

void UtilitaireMainWindow::on_btnStop_clicked()
{
    if(m_camera)
    {
        m_camera->abeCameraStopCapture();
    }
}

void UtilitaireMainWindow::drawRect(QRect rect)
{
    ui->camFullView->abeCameraWidgetDrawRect(rect);
}

///////////////////////////////////////////////////////////////////////////////////////
///                         Common Functions
///////////////////////////////////////////////////////////////////////////////////////

void UtilitaireMainWindow::slotSessionAuthenticated(bool enable)
{
    if(enable)
    {
        m_nam->abeSSOLogin();
    }
    show();
    connect(m_nam, SIGNAL(ssoAuthStatus(int)), this,SLOT(setTitle(int)),Qt::UniqueConnection);
}

void UtilitaireMainWindow::slotChangeLang()
{
    QString lang = sender()->objectName();
    abeApp->removeTranslator(&m_qtTranslator);
    abeApp->removeTranslator(&m_myAppTranslator);

    m_qtTranslator.load("qt_" + lang, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    abeApp->installTranslator(&m_qtTranslator);

    m_myAppTranslator.load(abeApp->applicationName() + "_" + lang, "lang");
    abeApp->installTranslator(&m_myAppTranslator);
    ui->retranslateUi(this);
}

void UtilitaireMainWindow::setTitle(int authStatus)
{
    QString title = abeApp->getAbeApplicationLongName();
    if (authStatus == 1)
    {
        title.append(" -- "+abeApp->getAbeNetworkAccessManager()->abeGetSSOLogin());
    }
    setWindowTitle(title);
}

void UtilitaireMainWindow::slotSSOStatusChanged(int code)
{
    /* Insérez ici le code que vous voulez concernant la modification d'état de l'authentification
       code = -1 (erreur d'authentification), 0 (on ne sait pas, jamais tenté), ou 1 (authentifié) */
    setTitle(code);
}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnOpen_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Ouvrir" */

    m_previousWidget = ui->centralStackedWidget->currentWidget();
    ui->centralStackedWidget->setCurrentWidget(ui->pageBoxFileManager);
    ui->widgetAbulEduBoxFileManager->abeSetOpenOrSaveEnum(AbulEduBoxFileManagerV1::abeOpen);
    ui->widgetAbulEduBoxFileManager->abeRefresh(AbulEduBoxFileManagerV1::abePC);
}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnSave_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Sauvegarder" */

    m_previousWidget = ui->centralStackedWidget->currentWidget();
    ui->centralStackedWidget->setCurrentWidget(ui->pageBoxFileManager);
    ui->widgetAbulEduBoxFileManager->abeSetOpenOrSaveEnum(AbulEduBoxFileManagerV1::abeSave);
    ui->widgetAbulEduBoxFileManager->abeRefresh(AbulEduBoxFileManagerV1::abePC);
}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnPublish_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Publier" le cas échéant */

}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnExport_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Exporter" le cas échéant */

}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnSettings_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Paramètres" le cas échéant */

}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnHelp_clicked()
{
    /* Écrivez ici le code à exécuter au clic sur l'entrée de menu "Fichier - Aide" le cas échéant */

    m_previousWidget = ui->centralStackedWidget->currentWidget();
    ui->centralStackedWidget->setCurrentWidget(ui->pageAPropos);

}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnQuit_clicked()
{
    close();
}

void UtilitaireMainWindow::on_action_Authentification_triggered()
{
    m_nam->abeSSOLogout();
    m_nam->abeSSOLogin();
}

void UtilitaireMainWindow::slotFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation loc, QString name, bool success)
{
    QString emplacement;
    if (loc == AbulEduBoxFileManagerV1::abePC) {
        emplacement = trUtf8("votre ordinateur");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeBoxPerso) {
        emplacement = trUtf8("votre abeBox personnelle");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeBoxShare) {
        emplacement = trUtf8("une abeBox partagée");
    }
    else if (loc == AbulEduBoxFileManagerV1::abeMediatheque) {
        emplacement = trUtf8("AbulEdu-Médiathèque");
    }
    else {
        emplacement = trUtf8("un endroit inconnu");
    }

    QString message;
    if (success == true) {
        message = trUtf8("Votre fichier a été enregistré dans ")+emplacement;
        if (!name.isEmpty()) {
            message.append(trUtf8(" sous le nom : ")+name.split("/").last());
        }
    }
    else {
        message = trUtf8("Votre fichier n'a pas pu être enregistré...");
    }
    AbulEduMessageBoxV1* msgEnregistrement = new AbulEduMessageBoxV1(trUtf8("Enregistrement"), message, true, this->centralWidget());
    if (success == true) {
        msgEnregistrement->setWink();
    }
    if(m_isCloseRequested) {
        connect(msgEnregistrement,SIGNAL(signalAbeMessageBoxCloseOrHide()),SLOT(slotForceClose()));
    }
    msgEnregistrement->show();
    ui->centralStackedWidget->setCurrentWidget(ui->pagePrincipale);
    m_previousWidget = ui->pagePrincipale;
}

void UtilitaireMainWindow::slotSaveProjectConfirmed()
{

    //Sauvegarde dans le fichier abe de toutes les images
    QStringList fileList;
    //        fileList.append(nomFichier);

    //    m_abuledufile->abeFileSetSubject();
    //    m_abuledufile->abeFileSetTitle();

    //    m_abuledufile->abeFileSetLOM();
    //    m_abuledufile->abeFileSetLOMExport(false);
    m_abuledufile->abeFileExportPrepare(fileList,m_abuledufile->abeFileGetDirectoryTemp().absolutePath(),"abe");

    m_previousWidget = ui->centralStackedWidget->currentWidget();
    ui->centralStackedWidget->setCurrentWidget(ui->pageBoxFileManager);
    ui->widgetAbulEduBoxFileManager->abeRefresh(AbulEduBoxFileManagerV1::abePC);
}

void UtilitaireMainWindow::slotForceClose()
{
    deleteLater();
}

void UtilitaireMainWindow::showEvent ( QShowEvent * event )
{
#ifdef __ABULEDUTABLETTEV1__MODE__
    showFullScreen();
#else
    showNormal();
#endif
}

/**
  * L'Action Quitter de la barre de menu
  */
void UtilitaireMainWindow::closeEvent(QCloseEvent *e)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(!isWindowModified()){
        e->accept();
        return;
    }
    else
    {
        e->ignore();
        m_isCloseRequested = true;
        AbulEduMessageBoxV1* mb = new AbulEduMessageBoxV1(trUtf8("Fermeture"),trUtf8("Fermeture du programme.\nVoulez-vous sauvegarder?"), true, this->centralWidget());
        mb->abeSetModeEnum(AbulEduMessageBoxV1::abeAcceptCancelCloseButton);
        mb->show();
        connect(mb,SIGNAL(signalAbeMessageBoxOK()),SLOT(on_actionSaveProject_triggered()));
        connect(mb,SIGNAL(signalAbeMessageBoxCloseOrHide()),SLOT(slotForceClose()));
    }

}

void UtilitaireMainWindow::slotAfficheWidgetPrincipal()
{
    ui->centralStackedWidget->setCurrentWidget(ui->pagePrincipale);
    m_previousWidget = ui->pagePrincipale;
}

void UtilitaireMainWindow::on_abeMenuFeuilleBtnNew_clicked()
{
    /* Mettez ici le code que vous voulez pour la creation d'un nouveau document */
    /* Exemple de comportement par défaut, si le document en cours a été modifié
       on demande si on souhaite le sauvegarder */
    if(isWindowModified()) {
        AbulEduMessageBoxV1* msg = new AbulEduMessageBoxV1(trUtf8("Sauvegarder le projet en cours ?"),trUtf8("Voulez-vous sauvegarder le projet en cours avant d'en créer un nouveau ?"), true, this->centralWidget());
        msg->abeSetModeEnum(AbulEduMessageBoxV1::abeYesNoButton);
        connect(msg,SIGNAL(signalAbeMessageBoxYES()), this, SLOT(slotSauvegardeProjet()));
        connect(msg,SIGNAL(signalAbeMessageBoxNO()), this, SLOT(slotNewProjectConfirmed()));
        msg->show();
    }
    else {
        slotNewProjectConfirmed();
    }
}

void UtilitaireMainWindow::slotNewProjectConfirmed()
{
    setWindowModified(false);
    m_abuledufile->abeClean();
}
