/** Classe de base pour les mainwindows des logiciels AbulEdu (Abuleducamera)
  *
  * @see https://redmine.ryxeo.com/projects/abuledu-abuleducamera
  * @author 2014 Jean-Louis Frucot<frucot.jeanlouis@free.fr>
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/UtilitaireMainWindow
  * @author 2012 Philippe <philippe.cadaugade@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef UTILITAIREMAINWINDOW_H
#define UTILITAIREMAINWINDOW_H

/* Inclusion de classes Qt */
#include <QMainWindow>
#include <QDebug>
# include <QThread>
/* Inclusion de classes AbulEdu */
#include "abuleduapplicationv1.h"
#include "abuledufilev1.h"
#include "abuleduboxfilemanagerv1.h"
//#include "abuleducameraworker.h"

#include "abuleducameracontrollerv1.h"

#include "abuleducamerawidgetv1.h"
namespace Ui {
class UtilitaireMainWindow;
}

class UtilitaireMainWindow : public QMainWindow
{
    Q_OBJECT

public:
    /* Méthodes */
    /** Constructeur */
    explicit UtilitaireMainWindow(QWidget *parent = 0);

    /** Destructeur virtuel pour cause d'objectif de polymorphisme */
    virtual ~UtilitaireMainWindow();

    void createCameraController();
protected:
    /* Attributs */
    /** Variable membre qui permet de contrôler l'affichage des qDebug */
    bool                     m_localDebug;

private:
    /** Gestion de l'affichage de la fenetre pour les tablettes */
    void showEvent ( QShowEvent * event );

    /* Attributs */
    /** Pointeur vers l'interface graphique */
    Ui::UtilitaireMainWindow* ui;

    /** QTranslator destiné aux traductions Qt System */
    QTranslator              m_qtTranslator;

    /** QTranslator destiné aux traductions spécifiques du logiciel */
    QTranslator              m_myAppTranslator;

    /** Pointeur vers un l'AbulEduNetworkAccessManagerV1 de l'application, assignation dans le constructeur */
    AbulEduNetworkAccessManagerV1 *m_nam;

    /** Il faudra probablement lire et écrire des fichiers abuledu ... */
    QSharedPointer<AbulEduFileV1>  m_abuledufile;

    /** Pointeur pour savoir sur quel widget on était avant de passer à un autre ... pour y revenir quand on "ferme" le widget en cours */
    QWidget *m_previousWidget;

    bool m_isCloseRequested;

    ////////////////////////////////////////////////////////////////////////////////
    ///                    My Project
    ////////////////////////////////////////////////////////////////////////////////

    AbulEduCameraControllerV1 *m_camera;
    AbulEduCameraWorker *m_cameraWorker;
    QThread *m_camThread;
//    QTimer *m_camTimer;
    QTime m_time;
    int m_imagesCount;
    bool m_getImageReady;




public slots:
    /** Affiche l'UtilitaireMainWindow après éventuellement demande d'authentification
      * Connecté dans la méthode setMainWindow() de l'AbulEduSplashScreenV1 */
    void slotSessionAuthenticated(bool enable);

private slots:
    /** Opère le changement de langue désiré
      * Connecté aux entrées du menu Langues */
    void slotChangeLang();

    /** Actualise le nom de la fenêtre en fonction du nom du logiciel, du nom de l'exercice et du nom de l'utilisateur */
    void setTitle(int authStatus);

    /** Permet de gerer les changement d'états d'authentification sso
      * @param code est l'entier envoyé dans le signal ssoAuthStatus(int) de l'AbulEduNetworkAccessManagerV1
      * avec lequel ce slot est connecté */
    void slotSSOStatusChanged(int code);

    /** Code à exécuter au clic sur l'entrée de menu "Utilisateur - Authentification" */
    void on_action_Authentification_triggered();

    /** Code à exécuter quand on sauvegarde un fichier */
    void slotFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation loc, QString name, bool success);

    /** Force la fermture de l'application */
    void slotForceClose();

    void closeEvent(QCloseEvent *);
    void slotAfficheWidgetPrincipal();
    void slotNewProjectConfirmed();
    void slotSaveProjectConfirmed();

    /** Slots correspondant aux boutons du menu feuille v1 */
    void on_abeMenuFeuilleBtnNew_clicked();
    void on_abeMenuFeuilleBtnOpen_clicked();
    void on_abeMenuFeuilleBtnSave_clicked();
    void on_abeMenuFeuilleBtnPublish_clicked();
    void on_abeMenuFeuilleBtnExport_clicked();
    void on_abeMenuFeuilleBtnSettings_clicked();
    void on_abeMenuFeuilleBtnHelp_clicked();
    void on_abeMenuFeuilleBtnQuit_clicked();

    ////////////////////////////////////////////////////////////////////////////////
    ///                    My Project
    ////////////////////////////////////////////////////////////////////////////////

    void slotCroppedQImageFromCamReceived(QImage image);
    void slotFullQImageFromCamReceived(QImage image);
    void slotGetImageSize(int width, int height);
//    void slotGetFullImage();
    void slotCameraErrorHandler(int errorNumber);
    void slotCameraFPSHandler(int fpsAverage);

    void on_sbDeviceNumber_valueChanged(int deviceNumber);

    void on_cbResolution_currentIndexChanged(int index);

    void on_hsFPS_valueChanged(int value);

    void on_btnCapture_clicked();
    void on_btnStop_clicked();

    void drawRect(QRect rect);
signals:

    void signalSetDevice(int deviceNumber, int width = 9999, int height = 9999);
    void signalSetImageSize(int, int);
//    void signalGetFrame();
};

#endif // UTILITAIREMAINWINDOW_H

